@@ -72,6 +72,53 @@ static const struct {
       AU_DEV_ENCODED }, /* This can also be forced with the --spdif option */
 };
 
+@interface SessionManager : NSObject
+{
+    NSMutableSet *_registeredInstances;
+}
++ (SessionManager *)sharedInstance;
+- (void)addAoutInstance:(AoutWrapper *)wrapperInstance;
+- (NSInteger)removeAoutInstance:(AoutWrapper *)wrapperInstance;
+@end
+
+@implementation SessionManager
++ (SessionManager *)sharedInstance
+{
+    static SessionManager *sharedInstance = nil;
+    static dispatch_once_t pred;
+
+    dispatch_once(&pred, ^{
+        sharedInstance = [SessionManager new];
+    });
+
+    return sharedInstance;
+}
+
+- (instancetype)init
+{
+    self = [super init];
+    if (self) {
+        _registeredInstances = [[NSMutableSet alloc] init];
+    }
+    return self;
+}
+
+- (void)addAoutInstance:(AoutWrapper *)wrapperInstance
+{
+    @synchronized(_registeredInstances) {
+        [_registeredInstances addObject:wrapperInstance];
+    }
+}
+
+- (NSInteger)removeAoutInstance:(AoutWrapper *)wrapperInstance
+{
+    @synchronized(_registeredInstances) {
+        [_registeredInstances removeObject:wrapperInstance];
+        return _registeredInstances.count;
+    }
+}
+@end
+
 /*****************************************************************************
  * aout_sys_t: private audio output method descriptor
  *****************************************************************************
@@ -87,7 +134,7 @@ struct aout_sys_t
     /* The AudioUnit we use */
     AudioUnit au_unit;
     bool      b_muted;
-    bool      b_paused;
+    bool      b_stopped;
     bool      b_preferred_channels_set;
     enum au_dev au_dev;
 
@@ -294,9 +341,15 @@ avas_SetActive(audio_output_t *p_aout, bool active, NSUInteger options)
         ret = [instance setCategory:AVAudioSessionCategoryPlayback error:&error];
         ret = ret && [instance setMode:AVAudioSessionModeMoviePlayback error:&error];
         ret = ret && [instance setActive:YES withOptions:options error:&error];
+        [[SessionManager sharedInstance] addAoutInstance: p_sys->aoutWrapper];
+    } else {
+        NSInteger numberOfRegisteredInstances = [[SessionManager sharedInstance] removeAoutInstance: p_sys->aoutWrapper];
+        if (numberOfRegisteredInstances == 0) {
+            ret = [instance setActive:NO withOptions:options error:&error];
+        } else {
+            ret = true;
+        }
     }
-    else
-        ret = [instance setActive:NO withOptions:options error:&error];
 
     if (!ret)
     {
@@ -322,7 +375,7 @@ Pause (audio_output_t *p_aout, bool pause, mtime_t date)
      * multi-tasking, the multi-tasking view would still show a playing state
      * despite we are paused, same for lock screen */
 
-    if (pause == p_sys->b_paused)
+    if (pause == p_sys->b_stopped)
         return;
 
     OSStatus err;
@@ -348,8 +401,16 @@ Pause (audio_output_t *p_aout, bool pause, mtime_t date)
             }
         }
     }
-    p_sys->b_paused = pause;
+    p_sys->b_stopped = pause;
     ca_Pause(p_aout, pause, date);
+
+    /* Since we stopped the AudioUnit, we can't really recover the delay from
+     * the last playback. So it's better to flush everything now to avoid
+     * synchronization glitches when resuming from pause. The main drawback is
+     * that we loose 1-2 sec of audio when resuming. The order is important
+     * here, ca_Flush need to be called when paused. */
+    if (pause)
+        ca_Flush(p_aout, false);
 }
 
 static void
@@ -397,7 +458,7 @@ Stop(audio_output_t *p_aout)
 
     [[NSNotificationCenter defaultCenter] removeObserver:p_sys->aoutWrapper];
 
-    if (!p_sys->b_paused)
+    if (!p_sys->b_stopped)
     {
         err = AudioOutputUnitStop(p_sys->au_unit);
         if (err != noErr)
