@@ -1807,6 +1807,11 @@ static void Render( decoder_t *p_dec, mtime_t i_start, mtime_t i_stop )
     vlc_array_init( &timedtags );
 
     GetTimedTags( p_sys->p_root->p_child, i_start, i_stop, &timedtags );
+    if( timedtags.i_count == 0 )
+    {
+        vlc_array_clear( &timedtags );
+        return;
+    }
     qsort( timedtags.pp_elems, timedtags.i_count, sizeof(*timedtags.pp_elems), timedtagsArrayCmp );
 
     mtime_t i_substart = i_start;
