@@ -2,7 +2,7 @@
  * standardpanel.cpp : The "standard" playlist panel : just a treeview
  ****************************************************************************
  * Copyright © 2000-2010 VideoLAN
- * $Id: 4f04823aeef90ff6ea6d1b3093adf03d3dc64de8 $
+ * $Id: 3d6fe6e6a8b9b5d55e720d348fda47a42495247a $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -40,10 +40,12 @@
 #include "dialogs/playlist.hpp"                   /* Playlist Dialog */
 #include "dialogs/mediainfo.hpp"                  /* MediaInfoDialog */
 #include "util/qt_dirs.hpp"
+#include "util/imagehelper.hpp"
 
 #include <vlc_services_discovery.h>               /* SD_CMD_SEARCH */
 #include <vlc_intf_strings.h>                     /* POP_ */
 
+#define SPINNER_SIZE 32
 #define I_NEW_DIR \
     I_DIR_OR_FOLDER( N_("Create Directory"), N_( "Create Folder" ) )
 #define I_NEW_DIR_NAME \
@@ -71,6 +73,8 @@
 
 #include <assert.h>
 
+#define DROPZONE_SIZE 112
+
 /* local helper */
 inline QModelIndex popupIndex( QAbstractItemView *view );
 
@@ -97,11 +101,11 @@ StandardPLPanel::StandardPLPanel( PlaylistWidget *_parent,
     lastActivatedPLItemId     = -1;
 
     QList<QString> frames;
-    frames << ":/util/wait1";
-    frames << ":/util/wait2";
-    frames << ":/util/wait3";
-    frames << ":/util/wait4";
-    spinnerAnimation = new PixmapAnimator( this, frames );
+    frames << ":/util/wait1.svg";
+    frames << ":/util/wait2.svg";
+    frames << ":/util/wait3.svg";
+    frames << ":/util/wait4.svg";
+    spinnerAnimation = new PixmapAnimator( this, frames, SPINNER_SIZE, SPINNER_SIZE );
     CONNECT( spinnerAnimation, pixmapReady( const QPixmap & ), this, updateViewport() );
 
     /* Saved Settings */
@@ -144,7 +148,7 @@ void StandardPLPanel::gotoPlayingItem()
 void StandardPLPanel::handleExpansion( const QModelIndex& index )
 {
     assert( currentView );
-    if( currentRootIndexPLId != -1 && currentRootIndexPLId != model->itemId( index.parent(), PLAYLIST_ID ) )
+    if( currentRootIndexPLId != -1 && currentRootIndexPLId != model->itemId( index.parent() ) )
         browseInto( index.parent() );
     currentView->scrollTo( index );
 }
@@ -186,27 +190,27 @@ bool StandardPLPanel::popup( const QPoint &point )
 
     /* Play/Stream/Info static actions */
 
-    ADD_MENU_ENTRY( QIcon( ":/menu/play" ), qtr(I_POP_PLAY),
+    ADD_MENU_ENTRY( QIcon( ":/toolbar/play_b.svg" ), qtr(I_POP_PLAY),
                     VLCModelSubInterface::ACTION_PLAY )
 
-    ADD_MENU_ENTRY( QIcon( ":/menu/pause" ), qtr("Pause"),
+    ADD_MENU_ENTRY( QIcon( ":/toolbar/pause_b.svg" ), qtr("Pause"),
                     VLCModelSubInterface::ACTION_PAUSE )
 
-    ADD_MENU_ENTRY( QIcon( ":/menu/stream" ), qtr(I_POP_STREAM),
+    ADD_MENU_ENTRY( QIcon( ":/menu/stream.svg" ), qtr(I_POP_STREAM),
                     VLCModelSubInterface::ACTION_STREAM )
 
     ADD_MENU_ENTRY( QIcon(), qtr(I_POP_SAVE),
                     VLCModelSubInterface::ACTION_SAVE );
 
-    ADD_MENU_ENTRY( QIcon( ":/menu/info" ), qtr(I_POP_INFO),
+    ADD_MENU_ENTRY( QIcon( ":/menu/info.svg" ), qtr(I_POP_INFO),
                     VLCModelSubInterface::ACTION_INFO );
 
     menu.addSeparator();
 
-    ADD_MENU_ENTRY( QIcon( ":/type/folder-grey" ), qtr(I_POP_EXPLORE),
+    ADD_MENU_ENTRY( QIcon( ":/type/folder-grey.svg" ), qtr(I_POP_EXPLORE),
                     VLCModelSubInterface::ACTION_EXPLORE );
 
-    QIcon addIcon( ":/buttons/playlist/playlist_add" );
+    QIcon addIcon( ":/buttons/playlist/playlist_add.svg" );
 
     ADD_MENU_ENTRY( addIcon, qtr(I_POP_NEWFOLDER),
                     VLCModelSubInterface::ACTION_CREATENODE )
@@ -236,10 +240,10 @@ bool StandardPLPanel::popup( const QPoint &point )
 
     /* Item removal */
 
-    ADD_MENU_ENTRY( QIcon( ":/buttons/playlist/playlist_remove" ), qtr(I_POP_DEL),
+    ADD_MENU_ENTRY( QIcon( ":/buttons/playlist/playlist_remove.svg" ), qtr(I_POP_DEL),
                     VLCModelSubInterface::ACTION_REMOVE );
 
-    ADD_MENU_ENTRY( QIcon( ":/toolbar/clear" ), qtr("Clear the playlist"),
+    ADD_MENU_ENTRY( QIcon( ":/toolbar/clear.svg" ), qtr("Clear the playlist"),
                     VLCModelSubInterface::ACTION_CLEAR );
 
     menu.addSeparator();
@@ -365,8 +369,7 @@ void StandardPLPanel::popupAction( QAction *action )
         uris = THEDP->showSimpleOpen();
         if ( uris.isEmpty() ) return;
         uris.sort();
-        foreach( const QString &file, uris )
-            a.uris << qtu( toURI( toNativeSeparators( file ) ) );
+        a.uris = uris;
         action->setData( QVariant::fromValue( a ) );
         model->action( action, list );
         break;
@@ -509,7 +512,7 @@ void StandardPLPanel::browseInto( const QModelIndex &index )
             currentView->scrollTo( newIndex );
 
         /* Store new rootindexid*/
-        currentRootIndexPLId = model->itemId( index, PLAYLIST_ID );
+        currentRootIndexPLId = model->itemId( index );
 
         model->ensureArtRequested( index );
     }
@@ -555,14 +558,24 @@ bool StandardPLPanel::eventFilter ( QObject *obj, QEvent * event )
         {
             QWidget *viewport = qobject_cast<QWidget *>( obj );
             QStylePainter painter( viewport );
-            QPixmap dropzone(":/dropzone");
+
+            QPixmap dropzone = ImageHelper::loadSvgToPixmap(":/dropzone.svg", DROPZONE_SIZE, DROPZONE_SIZE);
             QRect rect = viewport->geometry();
-            QSize size = rect.size() / 2 - dropzone.size() / 2;
+#if HAS_QT56
+            qreal scale = dropzone.devicePixelRatio();
+            QSize size = rect.size()  / 2 - dropzone.size() / (2 * scale);
+#else
+            QSize size = rect.size()  / 2 - dropzone.size() / 2;
+#endif
             rect.adjust( 0, size.height(), 0 , 0 );
             painter.drawItemPixmap( rect, Qt::AlignHCenter, dropzone );
             /* now select the zone just below the drop zone and let Qt center
                the text by itself */
-            rect.adjust( 0, dropzone.size().height() + 10, 0, 0 );
+#if HAS_QT56
+            rect.adjust( 0, dropzone.height() / scale + 10, 0, 0 );
+#else
+            rect.adjust( 0, dropzone.height() + 10, 0, 0 );
+#endif
             rect.setRight( viewport->geometry().width() );
             rect.setLeft( 0 );
             painter.drawItemText( rect,
@@ -582,10 +595,10 @@ bool StandardPLPanel::eventFilter ( QObject *obj, QEvent * event )
             {
                 QWidget *viewport = qobject_cast<QWidget *>( obj );
                 QStylePainter painter( viewport );
-                QPixmap *spinner = spinnerAnimation->getPixmap();
+                const QPixmap& spinner = spinnerAnimation->getPixmap();
                 QPoint point = viewport->geometry().center();
-                point -= QPoint( spinner->size().width() / 2, spinner->size().height() / 2 );
-                painter.drawPixmap( point, *spinner );
+                point -= QPoint( spinner.width() / 2, spinner.height() / 2 );
+                painter.drawPixmap( point, spinner );
             }
         }
     }
@@ -802,7 +815,7 @@ void StandardPLPanel::activate( const QModelIndex &index )
         else
         {
             playlist_Lock( THEPL );
-            playlist_item_t *p_item = playlist_ItemGetById( THEPL, model->itemId( index, PLAYLIST_ID ) );
+            playlist_item_t *p_item = playlist_ItemGetById( THEPL, model->itemId( index ) );
             if ( p_item )
             {
                 p_item->i_flags |= PLAYLIST_SUBITEM_STOP_FLAG;
