@@ -2,7 +2,7 @@
  * demux.c
  *****************************************************************************
  * Copyright (C) 1999-2004 VLC authors and VideoLAN
- * $Id: c5327520617ee3a612148606bebe274ed9b09c98 $
+ * $Id: 838ebe85911153ec4363d2466d84563b90bdf43c $
  *
  * Author: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -26,6 +26,7 @@
 #endif
 
 #include <assert.h>
+#include <limits.h>
 
 #include "demux.h"
 #include <libvlc.h>
@@ -35,31 +36,32 @@
 #include <vlc_modules.h>
 #include <vlc_strings.h>
 
-static bool SkipID3Tag( demux_t * );
-static bool SkipAPETag( demux_t *p_demux );
-
-struct demux_type
+typedef const struct
 {
-    char type[20];
-    char demux[8];
-};
+    char const key[20];
+    char const name[8];
 
-static int typecmp( const void *k, const void *t )
+} demux_mapping;
+
+static int demux_mapping_cmp( const void *k, const void *v )
 {
-    const char *key = k;
-    const struct demux_type *type = t;
-
-    return vlc_ascii_strcasecmp( key, type->type );
+    demux_mapping* entry = v;
+    return vlc_ascii_strcasecmp( k, entry->key );
 }
 
-static const char *demux_FromContentType(const char *mime)
+static demux_mapping* demux_lookup( char const* key,
+                                    demux_mapping* data, size_t size )
 {
-    static const struct demux_type types[] =
+    return bsearch( key, data, size, sizeof( *data ), demux_mapping_cmp );
+}
+
+static const char *demux_NameFromMimeType(const char *mime)
+{
+    static demux_mapping types[] =
     {   /* Must be sorted in ascending ASCII order */
         { "audio/aac",           "m4a"     },
         { "audio/aacp",          "m4a"     },
         { "audio/mpeg",          "mp3"     },
-        { "application/rss+xml", "podcast" },
         //{ "video/MP1S",          "es,mpgv" }, !b_force
         { "video/dv",            "rawdv"   },
         { "video/MP2P",          "ps"      },
@@ -67,11 +69,73 @@ static const char *demux_FromContentType(const char *mime)
         { "video/nsa",           "nsv"     },
         { "video/nsv",           "nsv"     },
     };
-    const struct demux_type *type;
+    demux_mapping *type = demux_lookup( mime, types, ARRAY_SIZE( types ) );
+    return (type != NULL) ? type->name : "any";
+}
 
-    type = bsearch( mime, types, sizeof (types) / sizeof (types[0]),
-                    sizeof (types[0]), typecmp );
-    return (type != NULL) ? type->demux : "any";
+static const char* DemuxNameFromExtension( char const* ext,
+                                           bool b_preparsing )
+{
+    /* NOTE: Add only file without any problems here and with strong detection:
+     * - no .mp3, .a52, ...
+     *  - wav can't be added 'cause of a52 and dts in them as raw audio
+     */
+    static demux_mapping strong[] =
+    { /* NOTE: must be sorted in asc order */
+        { "aiff", "aiff" },
+        { "asf",  "asf" },
+        { "au",   "au" },
+        { "avi",  "avi" },
+        { "drc",  "dirac" },
+        { "dv",   "dv" },
+        { "flac", "flac" },
+        { "h264", "h264" },
+        { "kar", "smf" },
+        { "m3u",  "m3u" },
+        { "m4a",  "mp4" },
+        { "m4v",  "m4v" },
+        { "mid",  "smf" },
+        { "mka",  "mkv" },
+        { "mks",  "mkv" },
+        { "mkv",  "mkv" },
+        { "moov", "mp4" },
+        { "mov",  "mp4" },
+        { "mp4",  "mp4" },
+        { "nsv",  "nsv" },
+        { "oga",  "ogg" },
+        { "ogg",  "ogg" },
+        { "ogm",  "ogg" },
+        { "ogv",  "ogg" },
+        { "ogx",  "ogg" }, /*RFC5334*/
+        { "opus", "ogg" }, /*draft-terriberry-oggopus-01*/
+        { "pva",  "pva" },
+        { "rm",   "avformat" },
+        { "rmi",  "smf" },
+        { "spx",  "ogg" },
+        { "voc",  "voc" },
+        { "wma",  "asf" },
+        { "wmv",  "asf" },
+    };
+
+    /* Here, we don't mind if it does not work, it must be quick */
+    static demux_mapping quick[] =
+    { /* NOTE: shall be sorted in asc order */
+        { "mp3", "mpga" },
+        { "ogg", "ogg" },
+        { "wma", "asf" },
+    };
+
+    struct {
+        demux_mapping* data;
+        size_t size;
+
+    } lookup = {
+        .data = b_preparsing ? quick : strong,
+        .size = b_preparsing ? ARRAY_SIZE( quick ) : ARRAY_SIZE( strong )
+    };
+
+    demux_mapping* result = demux_lookup( ext, lookup.data, lookup.size );
+    return result ? result->name : NULL;
 }
 
 /*****************************************************************************
@@ -87,6 +151,46 @@ demux_t *demux_New( vlc_object_t *p_obj, const char *psz_name,
                               psz_location, s, out, false );
 }
 
+typedef struct demux_priv_t
+{
+    demux_t demux;
+    void (*destroy)(demux_t *);
+} demux_priv_t;
+
+static void demux_DestroyDemux(demux_t *demux)
+{
+    assert(demux->s != NULL);
+    vlc_stream_Delete(demux->s);
+}
+
+static void demux_DestroyAccessDemux(demux_t *demux)
+{
+    assert(demux->s == NULL);
+    (void) demux;
+}
+
+static void demux_DestroyDemuxFilter(demux_t *demux)
+{
+    assert(demux->p_next != NULL);
+    demux_Delete(demux->p_next);
+}
+
+static int demux_Probe(void *func, va_list ap)
+{
+    int (*probe)(vlc_object_t *) = func;
+    demux_t *demux = va_arg(ap, demux_t *);
+
+    /* Restore input stream offset (in case previous probed demux failed to
+     * to do so). */
+    if (vlc_stream_Tell(demux->s) != 0 && vlc_stream_Seek(demux->s, 0))
+    {
+        msg_Err(demux, "seek failure before probing");
+        return VLC_EGENERIC;
+    }
+
+    return probe(VLC_OBJECT(demux));
+}
+
 /*****************************************************************************
  * demux_NewAdvanced:
  *  if s is NULL then load a access_demux
@@ -97,16 +201,18 @@ demux_t *demux_NewAdvanced( vlc_object_t *p_obj, input_thread_t *p_parent_input,
                             const char *psz_location,
                             stream_t *s, es_out_t *out, bool b_preparsing )
 {
-    demux_t *p_demux = vlc_custom_create( p_obj, sizeof( *p_demux ), "demux" );
-    if( unlikely(p_demux == NULL) )
+    demux_priv_t *priv = vlc_custom_create(p_obj, sizeof (*priv), "demux");
+    if (unlikely(priv == NULL))
         return NULL;
 
+    demux_t *p_demux = &priv->demux;
+
     if( s != NULL && (!strcasecmp( psz_demux, "any" ) || !psz_demux[0]) )
-    {   /* Look up demux by Content-Type for hard to detect formats */
-        char *type = stream_ContentType( s );
+    {   /* Look up demux by mime-type for hard to detect formats */
+        char *type = stream_MimeType( s );
         if( type != NULL )
         {
-            psz_demux = demux_FromContentType( type );
+            psz_demux = demux_NameFromMimeType( type );
             free( type );
         }
     }
@@ -128,8 +234,9 @@ demux_t *demux_NewAdvanced( vlc_object_t *p_obj, input_thread_t *p_parent_input,
                  p_demux->psz_access, p_demux->psz_demux,
                  p_demux->psz_location, p_demux->psz_file );
 
-    p_demux->s          = s;
-    p_demux->out        = out;
+    p_demux->s              = s;
+    p_demux->out            = out;
+    p_demux->b_preparsing   = b_preparsing;
 
     p_demux->pf_demux   = NULL;
     p_demux->pf_control = NULL;
@@ -137,89 +244,25 @@ demux_t *demux_NewAdvanced( vlc_object_t *p_obj, input_thread_t *p_parent_input,
     p_demux->info.i_update = 0;
     p_demux->info.i_title  = 0;
     p_demux->info.i_seekpoint = 0;
-
-    /* NOTE: Add only file without any problems here and with strong detection:
-     * - no .mp3, .a52, ...
-     *  - wav can't be added 'cause of a52 and dts in them as raw audio
-     */
-    static const struct { char ext[5]; char demux[9]; } exttodemux[] =
-    {
-        { "aiff", "aiff" },
-        { "asf",  "asf" }, { "wmv",  "asf" }, { "wma",  "asf" },
-        { "avi",  "avi" },
-        { "au",   "au" },
-        { "flac", "flac" },
-        { "dv",   "dv" },
-        { "drc",  "dirac" },
-        { "m3u",  "m3u" },
-        { "mkv",  "mkv" }, { "mka",  "mkv" }, { "mks",  "mkv" },
-        { "mp4",  "mp4" }, { "m4a",  "mp4" }, { "mov",  "mp4" }, { "moov", "mp4" },
-        { "nsv",  "nsv" },
-        { "ogg",  "ogg" }, { "ogm",  "ogg" }, /* legacy Ogg */
-        { "oga",  "ogg" }, { "spx",  "ogg" }, { "ogv", "ogg" },
-        { "ogx",  "ogg" }, /*RFC5334*/
-        { "opus", "ogg" }, /*draft-terriberry-oggopus-01*/
-        { "pva",  "pva" },
-        { "rm",   "avformat" },
-        { "m4v",  "m4v" },
-        { "h264", "h264" },
-        { "voc",  "voc" },
-        { "mid",  "smf" }, { "rmi",  "smf" }, { "kar", "smf" },
-        { "",  "" },
-    };
-    /* Here, we don't mind if it does not work, it must be quick */
-    static const struct { char ext[4]; char demux[5]; } exttodemux_quick[] =
-    {
-        { "mp3", "mpga" },
-        { "ogg", "ogg" },
-        { "wma", "asf" },
-        { "", "" }
-    };
+    priv->destroy = s ? demux_DestroyDemux : demux_DestroyAccessDemux;
 
     if( s != NULL )
     {
-        const char *psz_ext;
-        const char *psz_module = p_demux->psz_demux;
+        const char *psz_module = NULL;
 
-        if( !strcmp(psz_module, "any") && p_demux->psz_file != NULL
-         && (psz_ext = strrchr( p_demux->psz_file, '.' )) != NULL )
+        if( !strcmp( p_demux->psz_demux, "any" ) && p_demux->psz_file )
         {
-            psz_ext++; // skip '.'
+            char const* psz_ext = strrchr( p_demux->psz_file, '.' );
 
-            if( !b_preparsing )
-            {
-                for( unsigned i = 0; exttodemux[i].ext[0]; i++ )
-                {
-                    if( !strcasecmp( psz_ext, exttodemux[i].ext ) )
-                    {
-                        psz_module = exttodemux[i].demux;
-                        break;
-                    }
-                }
-            }
-            else
-            {
-                for( unsigned i = 0; exttodemux_quick[i].ext[0]; i++ )
-                {
-                    if( !strcasecmp( psz_ext, exttodemux_quick[i].ext ) )
-                    {
-                        psz_module = exttodemux_quick[i].demux;
-                        break;
-                    }
-                }
-            }
+            if( psz_ext )
+                psz_module = DemuxNameFromExtension( psz_ext + 1, b_preparsing );
         }
 
-        /* ID3/APE tags will mess-up demuxer probing so we skip it here.
-         * ID3/APE parsers will called later on in the demuxer to access the
-         * skipped info. */
-        while (SkipID3Tag( p_demux ))
-          ;
-        SkipAPETag( p_demux );
+        if( psz_module == NULL )
+            psz_module = p_demux->psz_demux;
 
-        p_demux->p_module =
-            module_need( p_demux, "demux", psz_module,
-                         !strcmp( psz_module, p_demux->psz_demux ) );
+        p_demux->p_module = vlc_module_load(p_demux, "demux", psz_module,
+             !strcmp(psz_module, p_demux->psz_demux), demux_Probe, p_demux);
     }
     else
     {
@@ -240,118 +283,21 @@ error:
     return NULL;
 }
 
-demux_t *input_DemuxNew( vlc_object_t *obj, const char *access_name,
-                         const char *demux_name, const char *path,
-                         es_out_t *out, bool preparsing, input_thread_t *input )
-{
-    char *demux_var = NULL;
-
-    assert( access_name != NULL );
-    assert( demux_name != NULL );
-    assert( path != NULL );
-
-    if( demux_name[0] == '\0' )
-    {
-        /* special hack for forcing a demuxer with --demux=module
-         * (and do nothing with a list) */
-        demux_var = var_InheritString( obj, "demux" );
-        if( demux_var != NULL )
-        {
-            demux_name = demux_var;
-            msg_Dbg( obj, "specified demux: %s", demux_name );
-        }
-        else
-            demux_name = "any";
-    }
-
-    demux_t *demux = NULL;
-
-    if( preparsing )
-    {
-        if( strcasecmp( demux_name, "any" ) )
-            goto out;
-
-        msg_Dbg( obj, "preparsing %s://%s", access_name, path );
-    }
-    else /* Try access_demux first */
-        demux = demux_NewAdvanced( obj, input, access_name, demux_name, path,
-                                   NULL, out, false );
-
-    if( demux == NULL )
-    {   /* Then try a real access,stream,demux chain */
-        /* Create the stream_t */
-        stream_t *stream = NULL;
-        char *url;
-
-        if( likely(asprintf( &url, "%s://%s", access_name, path) >= 0) )
-        {
-            stream = stream_AccessNew( obj, input, preparsing, url );
-            free( url );
-        }
-
-        if( stream == NULL )
-        {
-            msg_Err( obj, "cannot access %s://%s", access_name, path );
-            goto out;
-        }
-
-        /* Add stream filters */
-        stream = stream_FilterAutoNew( stream );
-
-        char *filters = var_InheritString( obj, "stream-filter" );
-        if( filters != NULL )
-        {
-            stream = stream_FilterChainNew( stream, filters );
-            free( filters );
-        }
-
-        if( var_InheritBool( obj, "input-record-native" ) )
-            stream = stream_FilterChainNew( stream, "record" );
-
-        /* FIXME: Hysterical raisins. Access is not updated according to any
-         * redirect but path is. This does not make much sense. Probably the
-         * URL should be passed as a whole and demux_t.psz_access removed. */
-        if( stream->psz_url != NULL )
-        {
-            path = strstr( stream->psz_url, "://" );
-            if( path == NULL )
-            {
-                stream_Delete( stream );
-                goto out;
-            }
-            path += 3;
-        }
-
-        demux = demux_NewAdvanced( obj, input, access_name, demux_name, path,
-                                   stream, out, preparsing );
-        if( demux == NULL )
-        {
-            msg_Err( obj, "cannot parse %s://%s", access_name, path );
-            stream_Delete( stream );
-        }
-    }
-out:
-    free( demux_var );
-    return demux;
-}
-
 /*****************************************************************************
  * demux_Delete:
  *****************************************************************************/
 void demux_Delete( demux_t *p_demux )
 {
-    stream_t *s;
+    demux_priv_t *priv = (demux_priv_t *)p_demux;
 
     module_unneed( p_demux, p_demux->p_module );
+
+    priv->destroy(p_demux);
     free( p_demux->psz_file );
     free( p_demux->psz_location );
     free( p_demux->psz_demux );
     free( p_demux->psz_access );
-
-    s = p_demux->s;
     vlc_object_release( p_demux );
-    if( s != NULL )
-        stream_Delete( s );
 }
 
 #define static_control_match(foo) \
@@ -384,7 +330,7 @@ int demux_vaControl( demux_t *demux, int query, va_list args )
                 va_copy( ap, args );
                 ret = demux->pf_control( demux, query, args );
                 if( ret != VLC_SUCCESS )
-                    ret = stream_vaControl( demux->s, query, ap );
+                    ret = vlc_stream_vaControl( demux->s, query, ap );
                 va_end( ap );
                 return ret;
             }
@@ -408,7 +354,7 @@ int demux_vaControl( demux_t *demux, int query, va_list args )
 
                 if( demux_ControlInternal( demux, DEMUX_CAN_PAUSE,
                                            &can_pause ) )
-                    return stream_vaControl( demux->s, query, args );
+                    return vlc_stream_vaControl( demux->s, query, args );
 
                 /* The caller shall not pause if pause is unsupported. */
                 assert( can_pause );
@@ -434,7 +380,7 @@ int demux_vaControlHelper( stream_t *s,
     if( i_end < 0 )    i_end   = stream_Size( s );
     if( i_start < 0 )  i_start = 0;
     if( i_align <= 0 ) i_align = 1;
-    i_tell = stream_Tell( s );
+    i_tell = vlc_stream_Tell( s );
 
     static_control_match(CAN_PAUSE);
     static_control_match(CAN_CONTROL_PACE);
@@ -450,7 +396,7 @@ int demux_vaControlHelper( stream_t *s,
             bool *b = va_arg( args, bool * );
 
             if( (i_bitrate <= 0 && i_start >= i_end)
-             || stream_Control( s, STREAM_CAN_SEEK, b ) )
+             || vlc_stream_Control( s, STREAM_CAN_SEEK, b ) )
                 *b = false;
             break;
         }
@@ -461,7 +407,7 @@ int demux_vaControlHelper( stream_t *s,
         case DEMUX_GET_META:
         case DEMUX_GET_SIGNAL:
         case DEMUX_SET_PAUSE_STATE:
-            return stream_vaControl( s, i_query, args );
+            return vlc_stream_vaControl( s, i_query, args );
 
         case DEMUX_GET_LENGTH:
             pi64 = (int64_t*)va_arg( args, int64_t * );
@@ -498,7 +444,7 @@ int demux_vaControlHelper( stream_t *s,
             {
                 int64_t i_block = (f * ( i_end - i_start )) / i_align;
 
-                if( stream_Seek( s, i_start + i_block * i_align ) )
+                if( vlc_stream_Seek( s, i_start + i_block * i_align ) )
                 {
                     return VLC_EGENERIC;
                 }
@@ -511,7 +457,7 @@ int demux_vaControlHelper( stream_t *s,
             if( i_bitrate > 0 && i64 >= 0 )
             {
                 int64_t i_block = i64 * i_bitrate / INT64_C(8000000) / i_align;
-                if( stream_Seek( s, i_start + i_block * i_align ) )
+                if( vlc_stream_Seek( s, i_start + i_block * i_align ) )
                 {
                     return VLC_EGENERIC;
                 }
@@ -531,6 +477,18 @@ int demux_vaControlHelper( stream_t *s,
         case DEMUX_SET_ES:
         case DEMUX_GET_ATTACHMENTS:
         case DEMUX_CAN_RECORD:
+        case DEMUX_TEST_AND_CLEAR_FLAGS:
+        case DEMUX_GET_TITLE:
+        case DEMUX_GET_SEEKPOINT:
+        case DEMUX_NAV_ACTIVATE:
+        case DEMUX_NAV_UP:
+        case DEMUX_NAV_DOWN:
+        case DEMUX_NAV_LEFT:
+        case DEMUX_NAV_RIGHT:
+        case DEMUX_NAV_POPUP:
+        case DEMUX_NAV_MENU:
+        case DEMUX_FILTER_ENABLE:
+        case DEMUX_FILTER_DISABLE:
             return VLC_EGENERIC;
 
         case DEMUX_SET_TITLE:
@@ -538,7 +496,7 @@ int demux_vaControlHelper( stream_t *s,
         case DEMUX_SET_RECORD_STATE:
             assert(0);
         default:
-            msg_Err( s, "unknown query in demux_vaControlDefault" );
+            msg_Err( s, "unknown query 0x%x in %s", i_query, __func__ );
             return VLC_EGENERIC;
     }
     return VLC_SUCCESS;
@@ -559,13 +517,11 @@ decoder_t *demux_PacketizerNew( demux_t *p_demux, es_format_t *p_fmt, const char
     }
     p_fmt->b_packetized = false;
 
-    p_packetizer->pf_decode_audio = NULL;
-    p_packetizer->pf_decode_video = NULL;
-    p_packetizer->pf_decode_sub = NULL;
+    p_packetizer->pf_decode = NULL;
     p_packetizer->pf_packetize = NULL;
 
     p_packetizer->fmt_in = *p_fmt;
-    es_format_Init( &p_packetizer->fmt_out, UNKNOWN_ES, 0 );
+    es_format_Init( &p_packetizer->fmt_out, p_fmt->i_cat, 0 );
 
     p_packetizer->p_module = module_need( p_packetizer, "packetizer", NULL, false );
     if( !p_packetizer->p_module )
@@ -590,69 +546,113 @@ void demux_PacketizerDestroy( decoder_t *p_packetizer )
     vlc_object_release( p_packetizer );
 }
 
-static bool SkipID3Tag( demux_t *p_demux )
+unsigned demux_TestAndClearFlags( demux_t *p_demux, unsigned flags )
 {
-    const uint8_t *p_peek;
-    uint8_t version, revision;
-    int i_size;
-    int b_footer;
+    unsigned update = flags;
 
-    if( !p_demux->s )
-        return false;
+    if ( demux_Control( p_demux, DEMUX_TEST_AND_CLEAR_FLAGS, &update ) == VLC_SUCCESS )
+        return update;
 
-    /* Get 10 byte id3 header */
-    if( stream_Peek( p_demux->s, &p_peek, 10 ) < 10 )
-        return false;
-
-    if( memcmp( p_peek, "ID3", 3 ) )
-        return false;
-
-    version = p_peek[3];
-    revision = p_peek[4];
-    b_footer = p_peek[5] & 0x10;
-    i_size = (p_peek[6]<<21) + (p_peek[7]<<14) + (p_peek[8]<<7) + p_peek[9];
-
-    if( b_footer ) i_size += 10;
-    i_size += 10;
-
-    /* Skip the entire tag */
-    if( stream_Read( p_demux->s, NULL, i_size ) < i_size )
-        return false;
-
-    msg_Dbg( p_demux, "ID3v2.%d revision %d tag found, skipping %d bytes",
-             version, revision, i_size );
-    return true;
-}
-static bool SkipAPETag( demux_t *p_demux )
-{
-    const uint8_t *p_peek;
-    int i_version;
-    int i_size;
-    uint32_t flags;
-
-    if( !p_demux->s )
-        return false;
-
-    /* Get 32 byte ape header */
-    if( stream_Peek( p_demux->s, &p_peek, 32 ) < 32 )
-        return false;
-
-    if( memcmp( p_peek, "APETAGEX", 8 ) )
-        return false;
-
-    i_version = GetDWLE( &p_peek[8] );
-    flags = GetDWLE( &p_peek[8+4+4] );
-    if( ( i_version != 1000 && i_version != 2000 ) || !( flags & (1<<29) ) )
-        return false;
-
-    i_size = GetDWLE( &p_peek[8+4] ) + ( (flags&(1<<30)) ? 32 : 0 );
-
-    /* Skip the entire tag */
-    if( stream_Read( p_demux->s, NULL, i_size ) < i_size )
-        return false;
-
-    msg_Dbg( p_demux, "AP2 v%d tag found, skipping %d bytes",
-             i_version/1000, i_size );
-    return true;
+    update = p_demux->info.i_update & flags;
+    p_demux->info.i_update &= ~flags;
+    return update;
 }
 
+int demux_GetTitle( demux_t *p_demux )
+{
+    int i_title;
+    if ( demux_Control( p_demux, DEMUX_GET_TITLE, &i_title ) == VLC_SUCCESS )
+        return i_title;
+    return p_demux->info.i_title;
+}
+
+int demux_GetSeekpoint( demux_t *p_demux )
+{
+    int i_seekpoint;
+    if ( demux_Control( p_demux, DEMUX_GET_SEEKPOINT, &i_seekpoint ) == VLC_SUCCESS  )
+        return i_seekpoint;
+    return p_demux->info.i_seekpoint;
+}
+
+static demux_t *demux_FilterNew( demux_t *p_next, const char *p_name )
+{
+    demux_priv_t *priv = vlc_custom_create(p_next, sizeof (*priv), "demux_filter");
+    if (unlikely(priv == NULL))
+        return NULL;
+
+    demux_t *p_demux = &priv->demux;
+
+    p_demux->p_next       = p_next;
+    p_demux->p_input      = NULL;
+    p_demux->p_sys        = NULL;
+    p_demux->psz_access   = NULL;
+    p_demux->psz_demux    = NULL;
+    p_demux->psz_location = NULL;
+    p_demux->psz_file     = NULL;
+    p_demux->out          = NULL;
+    priv->destroy         = demux_DestroyDemuxFilter;
+    p_demux->p_module =
+        module_need( p_demux, "demux_filter", p_name, p_name != NULL );
+
+    if( p_demux->p_module == NULL )
+        goto error;
+
+    return p_demux;
+error:
+    vlc_object_release( p_demux );
+    return NULL;
+}
+
+demux_t *demux_FilterChainNew( demux_t *p_demux, const char *psz_chain )
+{
+    if( !psz_chain || !*psz_chain )
+        return NULL;
+
+    char *psz_parser = strdup(psz_chain);
+    if(!psz_parser)
+        return NULL;
+
+    /* parse chain */
+    while(psz_parser)
+    {
+        config_chain_t *p_cfg;
+        char *psz_name;
+        char *psz_rest_chain = config_ChainCreate( &psz_name, &p_cfg, psz_parser );
+        free( psz_parser );
+        psz_parser = psz_rest_chain;
+
+        demux_t *filter = demux_FilterNew(p_demux, psz_name);
+        if (filter != NULL)
+            p_demux = filter;
+
+        free(psz_name);
+        config_ChainDestroy(p_cfg);
+    }
+
+    return p_demux;
+}
+
+static bool demux_filter_enable_disable( demux_t *p_demux_chain,
+                                          const char* psz_demux, bool b_enable )
+{
+    demux_t *p_demux = p_demux_chain;
+
+     if( strcmp( module_get_name( p_demux->p_module, false ), psz_demux) == 0 ||
+         strcmp( module_get_name( p_demux->p_module, true ), psz_demux ) == 0 )
+     {
+        demux_Control( p_demux,
+                       b_enable ? DEMUX_FILTER_ENABLE : DEMUX_FILTER_DISABLE );
+        return true;
+    }
+    return false;
+}
+
+bool demux_FilterEnable( demux_t *p_demux_chain, const char* psz_demux )
+{
+    return demux_filter_enable_disable( p_demux_chain, psz_demux, true );
+}
+
+bool demux_FilterDisable( demux_t *p_demux_chain, const char* psz_demux )
+{
+    return demux_filter_enable_disable( p_demux_chain, psz_demux, false );
+}
