@@ -2,7 +2,7 @@
  * thread.c : Playlist management functions
  *****************************************************************************
  * Copyright © 1999-2008 VLC authors and VideoLAN
- * $Id: 2d54850b3c8a8a79e08c3aac7a3a41926d3f81c1 $
+ * $Id: 8f2d1ca87609c77a335e00ccd0dfb86c85a32857 $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Clément Stenac <zorglub@videolan.org>
@@ -33,6 +33,7 @@
 #include <vlc_interface.h>
 #include <vlc_playlist.h>
 #include <vlc_rand.h>
+#include <vlc_renderer_discovery.h>
 #include "playlist_internal.h"
 
 /*****************************************************************************
@@ -96,16 +97,15 @@ static int InputEvent( vlc_object_t *p_this, char const *psz_cmd,
     VLC_UNUSED(p_this); VLC_UNUSED(psz_cmd); VLC_UNUSED(oldval);
     playlist_t *p_playlist = p_data;
 
-    if( newval.i_int != INPUT_EVENT_STATE &&
-        newval.i_int != INPUT_EVENT_DEAD )
-        return VLC_SUCCESS;
+    if( newval.i_int == INPUT_EVENT_DEAD )
+    {
+        playlist_private_t *sys = pl_priv(p_playlist);
 
-    PL_LOCK;
-
-    /* XXX: signaling while not changing any parameter... suspicious... */
-    vlc_cond_signal( &pl_priv(p_playlist)->signal );
-
-    PL_UNLOCK;
+        PL_LOCK;
+        sys->request.input_dead = true;
+        vlc_cond_signal( &sys->signal );
+        PL_UNLOCK;
+    }
     return VLC_SUCCESS;
 }
 
@@ -195,6 +195,7 @@ static bool PlayItem( playlist_t *p_playlist, playlist_item_t *p_item )
 {
     playlist_private_t *p_sys = pl_priv(p_playlist);
     input_item_t *p_input = p_item->p_input;
+    vlc_renderer_item_t *p_renderer;
 
     PL_ASSERT_LOCKED;
 
@@ -202,11 +203,22 @@ static bool PlayItem( playlist_t *p_playlist, playlist_item_t *p_item )
 
     p_item->i_nb_played++;
     set_current_status_item( p_playlist, p_item );
+    p_renderer = p_sys->p_renderer;
+    /* Retain the renderer now to avoid it to be released by
+     * playlist_SetRenderer when we exit the locked scope. If the last reference
+     * was to be released, we would use a dangling pointer */
+    if( p_renderer )
+        vlc_renderer_item_hold( p_renderer );
     assert( p_sys->p_input == NULL );
     PL_UNLOCK;
 
+    libvlc_MetadataCancel( p_playlist->obj.libvlc, p_item );
+
     input_thread_t *p_input_thread = input_Create( p_playlist, p_input, NULL,
-                                                   p_sys->p_input_resource );
+                                                   p_sys->p_input_resource,
+                                                   p_renderer );
+    if( p_renderer )
+        vlc_renderer_item_release( p_renderer );
     if( likely(p_input_thread != NULL) )
     {
         var_AddCallback( p_input_thread, "intf-event",
@@ -229,7 +241,7 @@ static bool PlayItem( playlist_t *p_playlist, playlist_item_t *p_item )
     if( !b_has_art || strncmp( psz_arturl, "attachment://", 13 ) )
     {
         PL_DEBUG( "requesting art for new input thread" );
-        libvlc_ArtRequest( p_playlist->p_libvlc, p_input, META_REQUEST_OPTION_NONE );
+        libvlc_ArtRequest( p_playlist->obj.libvlc, p_input, META_REQUEST_OPTION_NONE );
     }
     free( psz_arturl );
 
@@ -371,34 +383,17 @@ static playlist_item_t *NextItem( playlist_t *p_playlist )
             msg_Dbg( p_playlist,"repeating item" );
             return get_current_status_item( p_playlist );
         }
-        if( b_playstop )
+        if( b_playstop && get_current_status_item( p_playlist ) )
         {
             msg_Dbg( p_playlist,"stopping (play and stop)" );
             return NULL;
         }
 
         /* */
-        if( get_current_status_item( p_playlist ) )
-        {
-            playlist_item_t *p_parent = get_current_status_item( p_playlist );
-            while( p_parent )
-            {
-                if( p_parent->i_flags & PLAYLIST_SKIP_FLAG )
-                {
-                    msg_Dbg( p_playlist, "blocking item, stopping") ;
-                    return NULL;
-                }
-                p_parent = p_parent->p_parent;
-            }
-        }
 
         PL_DEBUG( "changing item without a request (current %i/%i)",
                   p_playlist->i_current_index, p_playlist->current.i_size );
         /* Can't go to next from current item */
-        if( get_current_status_item( p_playlist ) &&
-            get_current_status_item( p_playlist )->i_flags & PLAYLIST_SKIP_FLAG )
-            return NULL;
-
         if( p_sys->b_reset_currently_playing )
             ResetCurrentlyPlaying( p_playlist,
                                    get_current_status_item( p_playlist ) );
@@ -422,49 +417,50 @@ static playlist_item_t *NextItem( playlist_t *p_playlist )
             return NULL;
 
         p_new = ARRAY_VAL( p_playlist->current, p_playlist->i_current_index );
-        /* The new item can't be autoselected  */
-        if( p_new != NULL && p_new->i_flags & PLAYLIST_SKIP_FLAG )
-            return NULL;
     }
     return p_new;
 }
 
-static void LoopInput( playlist_t *p_playlist )
+static bool LoopInput( playlist_t *p_playlist )
 {
     playlist_private_t *p_sys = pl_priv(p_playlist);
     input_thread_t *p_input = p_sys->p_input;
 
     assert( p_input != NULL );
 
-    if( p_sys->request.b_request || p_sys->killed )
+    /* Wait for input to end or be stopped */
+    while( !p_sys->request.input_dead )
     {
-        PL_DEBUG( "incoming request - stopping current input" );
-        input_Stop( p_input );
-    }
-
-    switch( var_GetInteger( p_input, "state" ) )
-    {
-    case END_S:
-    case ERROR_S:
-    /* This input is dead. Remove it ! */
-        p_sys->p_input = NULL;
-        PL_DEBUG( "dead input" );
-        PL_UNLOCK;
-
-        var_SetAddress( p_playlist, "input-current", NULL );
-
-        /* WARNING: Input resource manipulation and callback deletion are
-         * incompatible with the playlist lock. */
-        if( !var_InheritBool( p_input, "sout-keep" ) )
-            input_resource_TerminateSout( p_sys->p_input_resource );
-        var_DelCallback( p_input, "intf-event", InputEvent, p_playlist );
-
-        input_Close( p_input );
-        PL_LOCK;
-        break;
-    default:
+        if( p_sys->request.b_request || p_sys->killed )
+        {
+            PL_DEBUG( "incoming request - stopping current input" );
+            input_Stop( p_input );
+        }
         vlc_cond_wait( &p_sys->signal, &p_sys->lock );
     }
+
+    input_item_t *item = input_GetItem(p_input);
+    assert(item);
+    bool ok = !input_item_HasErrorWhenReading(item);
+
+    /* This input is dead. Remove it ! */
+    PL_DEBUG( "dead input" );
+    p_sys->p_input = NULL;
+    p_sys->request.input_dead = false;
+    PL_UNLOCK;
+
+    var_SetAddress( p_playlist, "input-current", NULL );
+
+    /* WARNING: Input resource manipulation and callback deletion are
+     * incompatible with the playlist lock. */
+    if( !var_InheritBool( p_input, "sout-keep" ) )
+        input_resource_TerminateSout( p_sys->p_input_resource );
+    var_DelCallback( p_input, "intf-event", InputEvent, p_playlist );
+
+    input_Close( p_input );
+    PL_LOCK;
+
+    return ok;
 }
 
 static bool Next( playlist_t *p_playlist )
@@ -485,6 +481,7 @@ static void *Thread ( void *data )
 {
     playlist_t *p_playlist = data;
     playlist_private_t *p_sys = pl_priv(p_playlist);
+    bool played = false;
 
     PL_LOCK;
     while( !p_sys->killed )
@@ -498,20 +495,31 @@ static void *Thread ( void *data )
             continue;
         }
 
+        /* Playlist in running state */
         while( !p_sys->killed && Next( p_playlist ) )
-        {   /* Playlist in running state */
-            assert(p_sys->p_input != NULL);
+        {
+            bool ok = LoopInput( p_playlist );
+            if (ok)
+                p_sys->i_consecutive_errors = 0;
+            else
+            {
+                if (p_sys->i_consecutive_errors < 6)
+                    p_sys->i_consecutive_errors++;
 
-            do
-                LoopInput( p_playlist );
-            while( p_sys->p_input != NULL );
+                int slowdown = 1 << p_sys->i_consecutive_errors;
+                /* 100ms, 200ms, 400ms, 800ms, 1.6s, 3.2s */
+                mtime_t deadline = mdate() + slowdown * 100000L; /* usecs */
+                vlc_cond_timedwait(&p_sys->signal, &p_sys->lock, deadline);
+            }
+            played = true;
         }
 
+        /* Playlist stopping */
         msg_Dbg( p_playlist, "nothing to play" );
-        if( var_InheritBool( p_playlist, "play-and-exit" ) )
+        if( played && var_InheritBool( p_playlist, "play-and-exit" ) )
         {
             msg_Info( p_playlist, "end of playlist, exiting" );
-            libvlc_Quit( p_playlist->p_libvlc );
+            libvlc_Quit( p_playlist->obj.libvlc );
         }
 
         /* Destroy any video display now (XXX: ugly hack) */
